@@ -15,6 +15,7 @@ from alibi.utils.lang_model import LanguageModel
 
 from alibi.api.interfaces import Explainer, Explanation
 from alibi.api.defaults import DEFAULT_META_ANCHOR, DEFAULT_DATA_ANCHOR
+from alibi.exceptions import AlibiPredictorCallException, AlibiPredictorReturnTypeError
 from .anchor_base import AnchorBaseBeam
 from .anchor_explanation import AnchorExplanation
 
@@ -39,10 +40,10 @@ def _load_spacy_lexeme_prob(nlp: 'spacy.language.Language'):
             # spacy 2.3.0 moved lexeme_prob into a different package `spacy_lookups_data`
             # https://github.com/explosion/spaCy/issues/5638
             try:
-                table = nlp.vocab.lookups_extra.get_table('lexeme_prob')
+                table = nlp.vocab.lookups_extra.get_table('lexeme_prob')  # type: ignore[attr-defined]
                 # remove the default empty table
                 if table == dict():
-                    nlp.vocab.lookups_extra.remove_table('lexeme_prob')
+                    nlp.vocab.lookups_extra.remove_table('lexeme_prob')  # type: ignore[attr-defined]
             except KeyError:
                 pass
             finally:
@@ -79,8 +80,9 @@ class Neighbors(object):
         # list with spaCy lexemes in vocabulary
         # first if statement is a workaround due to some missing keys in models:
         # https://github.com/SeldonIO/alibi/issues/275#issuecomment-665017691
-        self.to_check = [self.nlp.vocab[w] for w in self.nlp.vocab.vectors if
-                         int(w) in self.nlp.vocab.strings and self.nlp.vocab[w].prob >= self.w_prob]
+        self.to_check = [self.nlp.vocab[w] for w in self.nlp.vocab.vectors
+                         if int(w) in self.nlp.vocab.strings and  # type: ignore[operator]
+                         self.nlp.vocab[w].prob >= self.w_prob]
         self.n_similar = n_similar
 
     def neighbors(self, word: str, tag: str, top_n: int) -> dict:
@@ -192,10 +194,10 @@ class UnknownSampler(AnchorTextSampler):
             Text to be processed.
         """
         # process text
-        processed = self.nlp(text)                                  # spaCy tokens for text
-        self.words = [x.text for x in processed]                    # list with words in text
-        self.positions = [x.idx for x in processed]                 # positions of words in text
-        self.punctuation = [x for x in processed if x.is_punct]     # list with punctuation in text
+        processed = self.nlp(text)  # spaCy tokens for text
+        self.words = [x.text for x in processed]  # list with words in text
+        self.positions = [x.idx for x in processed]  # positions of words in text
+        self.punctuation = [x for x in processed if x.is_punct]  # list with punctuation in text
 
         # set dtype
         self.set_data_type()
@@ -292,10 +294,10 @@ class SimilaritySampler(AnchorTextSampler):
         text
           Text to be processed.
         """
-        processed = self.nlp(text)                                  # spaCy tokens for text
-        self.words = [x.text for x in processed]                    # list with words in text
-        self.positions = [x.idx for x in processed]                 # positions of words in text
-        self.punctuation = [x for x in processed if x.is_punct]     # punctuation in text
+        processed = self.nlp(text)  # spaCy tokens for text
+        self.words = [x.text for x in processed]  # list with words in text
+        self.positions = [x.idx for x in processed]  # positions of words in text
+        self.punctuation = [x for x in processed if x.is_punct]  # punctuation in text
         self.tokens = processed
 
         # find similar words
@@ -440,7 +442,7 @@ class LanguageModelSampler(AnchorTextSampler):
     FILLING_PARALLEL = 'parallel'
     FILLING_AUTOREGRESSIVE = 'autoregressive'
 
-    def __init__(self, model: LanguageModel, perturb_opts: dict,):
+    def __init__(self, model: LanguageModel, perturb_opts: dict, ):
         """
         Initialize language model sampler. This sampler replaces words with the ones
         sampled according to the output distribution of the language model. There are
@@ -804,7 +806,7 @@ class LanguageModelSampler(AnchorTextSampler):
             Array containing num_samples elements. Each element is a perturbed sentence.
         """
         # chose the perturbation function
-        perturb_func = self._perturb_instances_parallel if filling == self.FILLING_PARALLEL\
+        perturb_func = self._perturb_instances_parallel if filling == self.FILLING_PARALLEL \
             else self._perturb_instance_ar
 
         # perturb instances
@@ -1078,7 +1080,7 @@ class LanguageModelSampler(AnchorTextSampler):
 
 
 DEFAULT_SAMPLING_UNKNOWN = {
-  "sample_proba": 0.5
+    "sample_proba": 0.5
 }
 """
 Default perturbation options for `unknown` sampling
@@ -1087,10 +1089,10 @@ Default perturbation options for `unknown` sampling
 """
 
 DEFAULT_SAMPLING_SIMILARITY = {
-  "sample_proba": 0.5,
-  "top_n": 100,
-  "temperature": 1.0,
-  "use_proba": False
+    "sample_proba": 0.5,
+    "top_n": 100,
+    "temperature": 1.0,
+    "use_proba": False
 }
 """
 Default perturbation options for `similarity` sampling
@@ -1105,16 +1107,16 @@ Default perturbation options for `similarity` sampling
 """
 
 DEFAULT_SAMPLING_LANGUAGE_MODEL = {
-  "filling": "parallel",
-  "sample_proba": 0.5,
-  "top_n": 100,
-  "temperature": 1.0,
-  "use_proba": False,
-  "frac_mask_templates": 0.1,
-  "batch_size_lm": 32,
-  "punctuation": string.punctuation,
-  "stopwords": [],
-  "sample_punctuation": False,
+    "filling": "parallel",
+    "sample_proba": 0.5,
+    "top_n": 100,
+    "temperature": 1.0,
+    "use_proba": False,
+    "frac_mask_templates": 0.1,
+    "batch_size_lm": 32,
+    "punctuation": string.punctuation,
+    "stopwords": [],
+    "sample_punctuation": False,
 }
 """
 Default perturbation options for `similarity` sampling
@@ -1175,7 +1177,7 @@ class AnchorText(Explainer):
     }
 
     def __init__(self,
-                 predictor: Callable,
+                 predictor: Callable[[List[str]], np.ndarray],
                  sampling_strategy: str = 'unknown',
                  nlp: Optional['spacy.language.Language'] = None,
                  language_model: Optional[LanguageModel] = None,
@@ -1187,7 +1189,7 @@ class AnchorText(Explainer):
         Parameters
         ----------
         predictor
-            A callable that takes a tensor of N data points as inputs and returns N outputs.
+            A callable that takes a list of text strings representing N data points as inputs and returns N outputs.
         sampling_strategy
             Perturbation distribution method.
             `unknown` - replaces words with UNKs.
@@ -1203,9 +1205,17 @@ class AnchorText(Explainer):
         kwargs
             Sampling parameters can be passed as `kwargs` depending on the `sampling_strategy`.
             Check default parameters defined in:
+
                 - :py:data:`alibi.explainers.anchor_text.DEFAULT_SAMPLING_UNKNOWN`
                 - :py:data:`alibi.explainers.anchor_text.DEFAULT_SAMPLING_SIMILARITY`
                 - :py:data:`alibi.explainers.anchor_text.DEFAULT_SAMPLING_LANGUAGE_MODEL`
+
+        Raises
+        ------
+        :py:class:`alibi.exceptions.AlibiPredictorCallException`
+            If calling `predictor` fails at runtime.
+        :py:class:`alibi.exceptions.AlibiPredictorReturnTypeError`
+            If the return type of `predictor` is not `np.ndarray`.
         """
         super().__init__(meta=copy.deepcopy(DEFAULT_META_ANCHOR))
         self._seed(seed)
@@ -1267,8 +1277,8 @@ class AnchorText(Explainer):
 
         # get default args
         default_args: dict = self.DEFAULTS[self.sampling_strategy]
-        perturb_opts: dict = deepcopy(default_args)                 # contains only the perturbation params
-        all_opts = deepcopy(default_args)                           # contains params + some potential incorrect params
+        perturb_opts: dict = deepcopy(default_args)  # contains only the perturbation params
+        all_opts = deepcopy(default_args)  # contains params + some potential incorrect params
 
         # compute common keys
         allowed_keys = set(perturb_opts.keys())
@@ -1506,7 +1516,19 @@ class AnchorText(Explainer):
     def _transform_predictor(self, predictor: Callable) -> Callable:
         # check if predictor returns predicted class or prediction probabilities for each class
         # if needed adjust predictor so it returns the predicted class
-        if np.argmax(predictor(['Hello world']).shape) == 0:
+        x = ['Hello world']
+        try:
+            prediction = predictor(x)
+        except Exception as e:
+            msg = f"Predictor failed to be called on x={x}. " \
+                  f"Check that `predictor` works with inputs of type List[str]."
+            raise AlibiPredictorCallException(msg) from e
+
+        if not isinstance(prediction, np.ndarray):
+            msg = f"Excepted predictor return type to be {np.ndarray} but got {type(prediction)}."
+            raise AlibiPredictorReturnTypeError(msg)
+
+        if np.argmax(prediction.shape) == 0:
             return predictor
         else:
             transformer = ArgmaxTransformer(predictor)
