@@ -17,10 +17,22 @@ from alibi.utils.tf import argmax_grad, argmin_grad, one_hot_grad, round_grad
 logger = logging.getLogger(__name__)
 
 
-class CounterFactualProto(Explainer, FitMixin):
+def CounterFactualProto(*args, **kwargs):
+    """
+    The class name `CounterFactualProto` is deprecated, please use `CounterfactualProto`.
+    """
+    # TODO: remove this function in an upcoming release
+    warning_msg = 'The class name `CounterFactualProto` is deprecated, please use `CounterfactualProto`.'
+    import warnings
+    warnings.warn(warning_msg, FutureWarning)
+
+    return CounterfactualProto(*args, **kwargs)
+
+
+class CounterfactualProto(Explainer, FitMixin):
 
     def __init__(self,
-                 predict: Union[Callable, tf.keras.Model],
+                 predict: Union[Callable[[np.ndarray], np.ndarray], tf.keras.Model],
                  shape: tuple,
                  kappa: float = 0.,
                  beta: float = .1,
@@ -644,7 +656,7 @@ class CounterFactualProto(Explainer, FitMixin):
 
     def fit(self, train_data: np.ndarray, trustscore_kwargs: dict = None, d_type: str = 'abdm',
             w: float = None, disc_perc: Sequence[Union[int, float]] = (25, 50, 75), standardize_cat_vars: bool = False,
-            smooth: float = 1., center: bool = True, update_feature_range: bool = True) -> "CounterFactualProto":
+            smooth: float = 1., center: bool = True, update_feature_range: bool = True) -> "CounterfactualProto":
         """
         Get prototypes for each class using the encoder or k-d trees.
         The prototypes are used for the encoder loss term or to calculate the optional trust scores.
